/*==================================================================
  ==================================================================
  ==	XMusic -- An Interactive Interface to Csound		  ==
  ==       Copyright (C) 1988 by George D. Drapeau                ==
  ==                  All Rights Reserved                         ==
  ==================================================================
  ==================================================================
*/
#include <X/Xlib.h>
#include <X/Xkeyboard.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "defs.h"
#include "externs.h"



ExitPgm()							    /* This function asks the user if s/he really... */
{								    /* ...wants to exit Music. */
  int		i;
  char		*sPtr;
  XButtonEvent	event;

  XMapWindow(exitWin);						    /* Show the Exit window */
  XClear(exitWin);						    /* Clear the window before drawing in it */
  for (i = 0, sPtr = exitLabel[0];				    /* Draw the text in the Exit window */
       i<EXITSIZE;
       i++, sPtr=exitLabel[i])
    {
      XText(exitWin,5,
	    (int)((exitFont->height * 1.5) * i + 5),
	    sPtr, strlen(sPtr)-1,
	    exitFont->id, exitFG, exitBG);
    }								    /* end for i... */
  XMaskEvent(ButtonPressed, &event);				    /* Listen only for ButtonPressed events */
  if (event.detail == LeftButton)				    /* Did the user really want to quit? */
    {
      CleanUp();						    /* Yes, clean up and exit Music */
      exit(0);
    }
  XUnmapWindow(exitWin);					    /* Hide the exit window */
}								    /* end function ExitPgm */




DoHelp()							    /* This function determines if the Help window... */
{								    /* ...needs to be hidden or shown.  If it needs... */
  static int	mapped = 0;					    /* ...to be shown, then the window is painted. */
  
  if (mapped)							    /* Is the Help window showing already? */
    {
      XUnmapWindow(helpWin);					    /* Yes, hide the Help window, and...  */
      mapped = 0;						    /* ...set the flag saying that the window is hidden */
    }
  else								    /* No, the Help window is not showing already */
    {
      XMapWindow(helpWin);					    /* Show the Help window */
      mapped = 1;						    /* Set the flag saying that the Help window is showing */
    }								    /* end else */
}								    /* end function DoHelp */



DoToolInfo()							    /* This function determines if the Info window... */
{								    /* ...needs to be hidden or shown.  If it needs... */
  if (infoMapped)						    /* Is the Info window showing already? */
    {
      XUnmapWindow(infoWin);					    /* Yes, hide the Info window, and... */
      infoMapped = 0;						    /* ...set the flag saying that the window is hidden */
      lastTool = NULL;						    /* Reset variable pointing to last tool looked at */
    }
  else								    /* No, the Info window is not showing already */
    {
      infoMapped = 1;						    /* Set the flag saying that the Info window is showing */
      ToolInfo();
    }
}								    /* end function DoToolInfo */




AddCharToString(c)
     char	c;
{
  if (inputStrLen >= MAXSTRLEN)
    return;
  inputStrLen++;
  inputStr[inputStrLen-1] = c;
}								    /* end function AddCharToString */


DeleteChar()
{
  if (inputStrLen > 0)
    inputStrLen--;
}								    /* end functionDeleteChar */


EraseString()
{
  inputStrLen = 0;
}								    /* end function EraseString */




EnterString(prompt)
     char	*prompt;
{
  inputStrLen = 0;
  XMapWindow(inputWin);
  DrawInputWindow(prompt);
  while(1)							    /* Enter an X event loop */
    {
      XKeyOrButtonEvent	event;
      int		nBytes;
      char		c, *key;

      XMaskEvent(KeyPressed|ButtonPressed, &event);		    /* Only interested in keys or buttons pressed */
      if (event.type == KeyPressed)				    /* Did the user type something? */
	{
	  key=XLookupMapping((XKeyPressedEvent *)&event, &nBytes);  /* Yes, what key did s/he type? */
	  if (nBytes == 0)					    /* User typed a key not in the keyboard mapping... */
	    continue;						    /* ...go to next iteration of the while loop */
	  c = *key;						    /* Store character that X function returned */
	  if (isprint(c))					    /* Did the user type a printable character ? */
	    {
	      AddCharToString(c);				    /* Yes, add the character to the input string */
	      DrawInputString();				    /* Redraw the string in the input window */
	    }							    /* end if alnum... */
	  else
	    if (c == '\r')					    /* Did user press RETURN? */
	      {
		XUnmapWindow(inputWin);
		return(inputStrLen);				    /* Yes, return the size of the string entered */
	      }							    /* end if c == '\r'... */
	  else
	    if ((c == '\b') || (c == '\177'))			    /* Did the user press BackSpace or Delete? */
	      {
		DeleteChar();					    /* Yes, delete the last character in the param name */
		DrawInputString();				    /* Redraw the string in the input window */
	      }
	}							    /* end if event.type == KeyPressed... */
      else
	if (event.type == ButtonPressed)			    /* Did the user click a mouse button? */
	  {
	    if (event.detail == LeftButton)			    /* Yes, did s/he click the button to confirm param name? */
	      {
		XUnmapWindow(inputWin);
		return(inputStrLen);				    /* Yes, return the size of the string entered */
	      }
	    else
	      {
		XUnmapWindow(inputWin);
		return(0);					    /* Leave this function without doing anything */
	      }							    /* end else... */
	  }							    /* end if event.type == ButtonPressed... */
    }								    /* end while... */
}								    /* end function EnterString */
