/*==================================================================
  ==================================================================
  ==	XMusic -- An Interactive Interface to Csound		  ==
  ==       Copyright (C) 1988 by George D. Drapeau                ==
  ==                  All Rights Reserved                         ==
  ==================================================================
  ==================================================================
*/
#include <X/Xlib.h>
#include <stdio.h>
#define USEBITMAPS
#include "defs.h"

extern char		*WhoAmI;
extern int		changesMade;
extern Tool		*toolList;


char *exitLabel[EXITSIZE] =					    /* Allocate space for the exit window's text */
{
  " Are You SURE You Want To Exit?  If So, Please ",
  " Press The LEFT Button One More Time To Exit,  ",
  " MIDDLE Or RIGHT To Return To XMusic.           " 
  };

char *helpLabel[HELPSIZE] =					    /* Allocate space for the help window's text */
{
  "             Help for XMusic",
  "-----------------------------------------------------------------",
  "Press 'q' to Quit XMusic.",
  "Press 'h' to Show/Hide This Help Window.",
  "Press 'i' to Get Information About The Tool Being Pointed To.",
  "Press 'r' to Remove The Tool Being Pointed To.",
  "Press 's' to Save the Diagram To a File.",
  "Press 'w' to Write the Diagram To a Csound File.",
  "Right Button : Disconnect Two Tools.",
  "Middle Button: Move A Tool.",
  "Left Button  :",
  "      In ToolPalette -- Select A Tool.",
  "      In Music Window -- Connect Two Tools.",
  "-----------------------------------------------------------------",
  "To Connect Two Tools, first point to a tool, then click the left",
  "button to select it.  Then point to the tool you wish to make ",
  "a connection with, and again click on the left button.",
  "To Disconnect Two Tools, first point to a tool, then click the Right",
  "button to select it.  Then point to the tool you wish to disconnect",
  "and again click on the right button."
  };

char *greeting = "Welcome to XMusic.  Press 'h' For Help...";


Window			musicWin, exitWin, helpWin, toolWin;
Window			iconWin, inputWin, infoWin;
FontInfo		*musicFont, *exitFont, *helpFont, *infoFont;
Pixmap			borderPix,bgndPix, musicPixmap, sineMap;
Pixmap			envMap, sumMap, ampMap, iconMap, paramMap;
Pixmap			noiseMap, eqMap;
Cursor			mCursor, tCursor;
int			bgndTile, inverse;
int			musicFG,musicBG, exitFG,exitBG, helpFG,helpBG;
int			mCursorFG,mCursorBG, tCursorFG,tCursorBG;
int			infoFG,infoBG,borderWidth;
short			topX=0,topY=0;				    /* Coordinates used by DrawPixmap and DrawMusicWin */
int			botX,botY,wid,hgt;			    /* Also used by DrawPixmap and DrawMusicWin */
OpaqueFrame		musicBorder;
char			geometry[32];				    /* User-supplied geometry spec for the Music window */
char			defGeometry[32];			    /* Default geometry spec if user didn't say */
Color			fgColor,bgColor,borderColor,cursorColor;    /* Structures to store user-specified colors */




CreateWindows()							    /* Calls the necessary functions to create... */
{								    /* ...windows used by XMusic */
  exitWin	=	CreateTextWin(exitLabel,EXITSIZE,	    /* Create the Exit window */
				      350,400,exitFont);
  helpWin	=	CreateTextWin(helpLabel,HELPSIZE,	    /* Create the Help window */
				      500,50,helpFont);
  toolWin	=	XCreateWindow(RootWindow,0,50,		    /* Create the tool palette window */
				      sine64_width+4,
				      NUMTOOLS*sine64_height+4,
				      borderWidth,
				      borderPix, bgndPix);
  inputWin	=	XCreateWindow(RootWindow,500,400,	    /* Create an input window for parameter names */
				      (musicFont->width * 35) + 10,
				      (musicFont->height * 5) + 25,
				      borderWidth,
				      borderPix,bgndPix);
  infoWin	=	XCreateWindow(RootWindow,500,500,	    /* Create the Info window */
				      (infoFont->width * (2 * INFOSTRLEN)) + 10,
				      (int)((infoFont->height * 1.5) * INFOSIZE + 10),
				      borderWidth,
				      borderPix, bgndPix);
  GetDiagramSize();						    /* Figure the best size for the Music window */

  musicWin	=	XCreate(greeting,			    /* Let X prompt user for window size */
				WhoAmI,geometry,defGeometry,
				&musicBorder,MINWIDTH,MINHEIGHT);
}								    /* end function CreateWindows */



GetXDefaults()							    /* This function is responsible for setting up all... */
{								    /* ...user-defined parameters.  They... */
  char	*option;						    /* ...are retrieved from the .Xdefaults file. */

  if ((option = XGetDefault(WhoAmI,"musicFont")) != NULL)
    musicFont = XOpenFont(option);
  else
    musicFont = XOpenFont("vtsingle");
  
  if ((option = XGetDefault(WhoAmI,"ExitFont")) != NULL)
    exitFont  = XOpenFont(option);
  else
    exitFont  = XOpenFont("helv12b");
  
  if ((option = XGetDefault(WhoAmI,"HelpFont")) != NULL)
    helpFont  = XOpenFont(option);
  else
    helpFont  = XOpenFont("serif.r.12");
  
  if ((option = XGetDefault(WhoAmI,"InfoFont")) != NULL)
    infoFont  = XOpenFont(option);
  else
    infoFont  = XOpenFont("serif.r.12");
  
  if ((option = XGetDefault(WhoAmI,"ReverseVideo")) != NULL)
    if (strcmp(option, "on") == 0)
      inverse = 1;


  if ((option = XGetDefault(WhoAmI,"Background")) == NULL)	    /* Did user specify an X default for bg color? */
    option = "black";						    /* Nope, choose black as the default */
  if (XParseColor(option,&bgColor)==0)				    /* Does X recognize the user's bg color choice? */
    {								    /* Nope, tell the user and exit. */
      fprintf(stderr,"%s: X does not recognize the color %s, using Black.\n",WhoAmI,option);
    }
  if (XGetHardwareColor(&bgColor)==0)				    /* Can the X server provide the color requested? */
      bgColor.pixel = BlackPixel;				    /* Nope, pick a default color to use */

  if ((option = XGetDefault(WhoAmI,"Foreground")) == NULL)	    /* Did user specify an X default for fg color? */
    option = "Yellow";						    /* Nope, choose white as the default */
  if (XParseColor(option,&fgColor)==0)				    /* Does X recognize the user's fg color choice? */
    {								    /* Nope, tell the user and exit. */
      fprintf(stderr,"%s: X does not recognize the color %s, using White.\n",WhoAmI,option);
    }
  if (XGetHardwareColor(&fgColor)==0)				    /* Can the X server provide the color requested? */
      fgColor.pixel = WhitePixel;				    /* Nope, pick a default color to use */

  if ((option = XGetDefault(WhoAmI,"Border")) == NULL)		    /* Did user specify an X default for border color? */
    option = "Red";						    /* Nope, choose white as the default */
  if (XParseColor(option,&borderColor)==0)			    /* Does X recognize the user's border color choice? */
    {								    /* Nope, tell the user and exit. */
      fprintf(stderr,"%s: X does not recognize the color %s, using White.\n",WhoAmI,option);
    }
  if (XGetHardwareColor(&borderColor)==0)			    /* Can the X server provide the color requested? */
      borderColor.pixel = fgColor.pixel;			    /* Nope, pick a default color to use */

  if ((option = XGetDefault(WhoAmI,"borderWidth")) != NULL)	    /* Did user specify an X default for border width? */
    borderWidth = atoi(option);					    /* Yes, set variable to user's choice */
  else
    borderWidth = 3;						    /* Nope, choose 3 as the default */

  if ((option = XGetDefault(WhoAmI,"Cursor")) == NULL)		    /* Did user specify an X default for cursor color? */
    option = "Red";						    /* Nope, choose white as the default */
  if (XParseColor(option,&cursorColor)==0)			    /* Does X recognize the user's cursor color choice? */
    {								    /* Nope, tell the user and exit. */
      fprintf(stderr,"%s: X does not recognize the color %s, using White.\n",WhoAmI,option);
    }
  if (XGetHardwareColor(&cursorColor)==0)			    /* Can the X server provide the color requested? */
      cursorColor.pixel = fgColor.pixel;			    /* Nope, pick a default color to use */
  

  if (inverse == 1)						    /* Is inverse video set? */
    {								    /* Yes, redefine values for border colors, etc. */
      bgndTile	= fgColor.pixel;
      borderColor.pixel = bgColor.pixel;
      musicFG	= exitFG = helpFG = infoFG = bgColor.pixel;
      musicBG	= exitBG = helpBG = infoBG = fgColor.pixel;
      mCursorFG = tCursorFG = bgColor.pixel;
      mCursorBG = tCursorBG = fgColor.pixel;
    }
  else								    /* Nope, inverse video was not set. Set default values */
    {
      bgndTile	= bgColor.pixel;
      musicFG	= exitFG = helpFG = infoFG = fgColor.pixel;
      musicBG	= exitBG = helpBG = infoBG = bgColor.pixel;
      mCursorFG = tCursorFG = cursorColor.pixel;
      mCursorBG = tCursorBG = bgndTile;
    }
  bgndPix		=	XMakeTile(bgndTile);
  borderPix		=	XMakeTile(borderColor.pixel);	    /* Make borders the color specified by the user */
  musicBorder.bdrwidth	=	borderWidth;			    /* Set up defaults for the Music window's border */
  musicBorder.border	=	borderPix;
  musicBorder.background=	XMakeTile(bgndTile);
}								    /* end function GetXDefaults */



CleanUp()
{
  XFreePixmap(musicPixmap);					    /* Free up the storage used by Music window's Pixmap */
  XCloseFont(musicFont);
  XCloseFont(exitFont);
  XCloseFont(helpFont);
  XCloseFont(infoFont);
  XFreeCursor(mCursor);
  XFreeCursor(tCursor);
  XFreePixmap(sumMap);
  XFreePixmap(envMap);
  XFreePixmap(sineMap);
  XFreePixmap(iconMap);
  XFreePixmap(paramMap);
  XFreePixmap(eqMap);
  if (changesMade)
    WriteDiagram();
}



Window	CreateTextWin(textPtr,numLines,x,y,fInfo)		    /* This function takes as arguments: */
     char	**textPtr;					    /* A pointer to an array of strings (+ number of strings) */
     int	numLines,x,y;					    /* An upper-left coordinate for a window,  */
     FontInfo	*fInfo;						    /* and a Font specification. */
{								    /* The function calculates the best size of... */
								    /* ...a window to contain the text array. */

  int		maxChars, winWidth, winHeight;
  Window	wID;
  char		*sPtr;

  maxChars = GetMaxStrLen(textPtr,numLines);			    /* Get the size of the longest line in the text array */
  winWidth = (maxChars * fInfo->width) + 10 + borderWidth;	    /* Calculate window width based on longest line-length */
  winHeight = numLines * (fInfo->height * 1.5) + 5;		    /* Calculate window height based on number of lines */
  wID = XCreateWindow(RootWindow, x, y,				    /* Create an X window based on the above information */
		      winWidth, winHeight,
		      borderWidth, borderPix, bgndPix);
  return(wID);							    /* Return the ID of the X window to the caller */
}								    /* end function CreateTextWin */


GetMaxStrLen(strArray,numStrings)				    /* This function takes an array of strings and... */
     char	*strArray[];					    /* ...returns the length of the longest string */
     int	numStrings;
{
  int	count, maxStrLen = 0;
  char	*str;

  for (str = strArray[0], count = 0;
       count < numStrings;
       str=strArray[count], count++)
    maxStrLen = max(maxStrLen, strlen(str));
  return(maxStrLen);
}								    /* end function GetMaxStrLen */



GetXResources()
{
  mCursor = XCreateCursor(mCursor_width,mCursor_height,
			  mCursor_bits,mCursor_bits,
			  mCursor_x_hot,mCursor_y_hot,
			  mCursorFG,mCursorBG,
			  GXcopy);
  tCursor = XCreateCursor(tCursor_width,tCursor_height,
			  tCursor_bits,tCursor_bits,
			  tCursor_x_hot,tCursor_y_hot,
			  tCursorFG,tCursorBG,
			  GXcopy);
  XDefineCursor(musicWin,mCursor);
  XDefineCursor(exitWin,mCursor);
  XDefineCursor(helpWin,mCursor);
  XDefineCursor(infoWin,mCursor);
  sineMap	= XMakePixmap(XStoreBitmap(sine64_width,sine64_height,sine64_bits),
			      musicFG, musicBG);
  envMap	= XMakePixmap(XStoreBitmap(env64_width,env64_height,env64_bits),
			      musicFG, musicBG);
  sumMap	= XMakePixmap(XStoreBitmap(sum_width,sum_height,sum_bits),
			      musicFG, musicBG);
  ampMap	= XMakePixmap(XStoreBitmap(amp64_width,amp64_height,amp64_bits),
			      musicFG, musicBG);
  iconMap	= XMakePixmap(XStoreBitmap(mIcon_width,mIcon_height,mIcon_bits),
			      musicFG, musicBG);
  paramMap	= XMakePixmap(XStoreBitmap(param64_width,param64_height,param64_bits),
			      musicFG, musicBG);
  noiseMap	= XMakePixmap(XStoreBitmap(noise64_width,noise64_height,noise64_bits),
			      musicFG, musicBG);
  eqMap		= XMakePixmap(XStoreBitmap(eq64_width,eq64_height,eq64_bits),
			      musicFG, musicBG);
  iconWin =XCreateWindow(RootWindow,65,0,mIcon_width,mIcon_height,
			 2,borderPix,bgndPix);
  XSetIconWindow(musicWin,iconWin);
}




GetDiagramSize()						    /* This function determines the best window size to... */
{								    /* ...contain the diagram the user is looking at */
  Tool	*tPtr;
  int	minX = TOOLSIZE, minY = TOOLSIZE, maxX = DEFWIDTH, maxY = DEFHEIGHT;
  
  if (!toolList)						    /* Is the tool list empty? */
    {
      sprintf(defGeometry,"=%dx%d+75+50",DEFWIDTH,DEFHEIGHT);	    /* Set default geometry for Music window */
      return;
    }
  for (tPtr = toolList;  tPtr;  tPtr = tPtr->next)		    /* Go through the list of tools and look at each... */
    {								    /* ...of the tools' coordinates.  Determine the... */
      minX = min(minX, tPtr->x);				    /* ...upper leftmost tool and the lower rightmost. */
      minY = min(minY, tPtr->y);
      maxX = max(maxX, tPtr->x);
      maxY = max(maxY, tPtr->y);
    }
  if (minX < (TOOLSIZE/2))					    /* Are any tools above the top of the window? */
    {
      maxX += ((TOOLSIZE) - minX);				    /* Adjust window size to accomodate moving the tools */
      for (tPtr = toolList;  tPtr;  tPtr = tPtr->next)		    /* Yes, push all tools down so all tools are... */
	tPtr->x += ((TOOLSIZE) - minX);				    /* ...shown completely */
    }
  if (minY < (TOOLSIZE/2))					    /* Are any tools to the left of the window? */
    {
      maxY += ((TOOLSIZE) - minY);				    /* Adjust window size to accomodate moving the tools */
      for (tPtr = toolList;  tPtr;  tPtr = tPtr->next)		    /* Yes, push all tools to the right so all... */
	tPtr->y += ((TOOLSIZE) - minY);				    /* ...tools are shown completely */
     }
  if (maxX > 16383)						    /* Is the diagram too big for X to make a window for? */
    {								    /* Yes, report the error and correct for it */
      printf("Sorry, your diagram is too large to be drawn completely.\n");
      maxX = 16383;
    }
  if (maxY > 16383)
    {
      printf("Sorry, your diagram is too large to be drawn completely.\n");
      maxY = 16383;
    }
  sprintf(defGeometry,"=%dx%d+75+50",maxX + (TOOLSIZE), maxY + (TOOLSIZE));
}								    /* end function GetDiagramSize */
